unit UFormMain;

{$mode delphi}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, Menus,
  ActnList, ExtCtrls, StdCtrls, UGame, UPersistentForm, UApplicationInfo;

type

  { TFormMain }

  TFormMain = class(TForm)
    MainMenu1: TMainMenu;
    MenuItem1: TMenuItem;
    MenuItem2: TMenuItem;
    MenuItem3: TMenuItem;
    MenuItem4: TMenuItem;
    MenuItemMovesHistory: TMenuItem;
    MenuItemTools: TMenuItem;
    MenuItemNew: TMenuItem;
    MenuItemExit: TMenuItem;
    MenuItemHelp: TMenuItem;
    MenuItemAbout: TMenuItem;
    MenuItemGame: TMenuItem;
    TimerDraw: TTimer;
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormPaint(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure TimerDrawTimer(Sender: TObject);
  private
    RedrawPending: Boolean;
  public
    procedure Redraw;
    procedure UpdateInterface;
  end;

var
  FormMain: TFormMain;

implementation

{$R *.lfm}

uses
  UCore;

{ TFormMain }

procedure TFormMain.FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if Core.Game.Running and not Core.Game.Moving then begin
    case Key of
      37: Core.Game.MoveAllAndUpdate(drLeft, True);
      38: Core.Game.MoveAllAndUpdate(drUp, True);
      39: Core.Game.MoveAllAndUpdate(drRight, True);
      40: Core.Game.MoveAllAndUpdate(drDown, True);
    end;
  end;
end;

procedure TFormMain.FormPaint(Sender: TObject);
begin
  Core.Game.Render(Canvas, Point(Width, Height - MainMenu1.Height));
end;

procedure TFormMain.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin
  Core.PersistentForm1.Save(Self);
end;

procedure TFormMain.FormCreate(Sender: TObject);
begin
end;

procedure TFormMain.FormDestroy(Sender: TObject);
begin
end;

procedure TFormMain.FormShow(Sender: TObject);
begin
  Core.PersistentForm1.RegistryContext := Core.ApplicationInfo1.GetRegistryContext;
  Core.PersistentForm1.Load(Self);
  UpdateInterface;
  if Core.Game.Board.GetEmptyTilesCount > Core.Game.Board.Size.X * Core.Game.Board.Size.Y -
    InitialTileCount then
    Core.Game.New;
end;

procedure TFormMain.TimerDrawTimer(Sender: TObject);
begin
  if RedrawPending then begin
    RedrawPending := False;
    Repaint;
  end;
end;

procedure TFormMain.Redraw;
begin
  RedrawPending := True;
end;

procedure TFormMain.UpdateInterface;
var
  I: Integer;
  ToolsVisible: Boolean;
begin
  MenuItemMovesHistory.Visible := Core.Game.RecordHistory;
  ToolsVisible := False;
  for I := 0 to MenuItemTools.Count - 1 do
    if MenuItemTools.Items[I].Visible then begin
      ToolsVisible := True;
      Break;
    end;
  MenuItemTools.Visible := ToolsVisible;
end;

end.

